
/* $Id: dialog.cpp 278 2012-01-27 10:35:57Z sausage $
 * EOSERV is released under the zlib license.
 * See LICENSE.txt for more info.
 */

#include "dialog.hpp"

#include "util.hpp"

#include "packet.hpp"

Dialog::Dialog()
{

}

void Dialog::AddPage(const std::string& text)
{
	this->pages.push_back(text);
}

void Dialog::AddLink(int id, const std::string& text)
{
	this->links.insert(std::make_pair(id, text));
}

bool Dialog::CheckLink(int id) const
{
	return this->links.find(id) != this->links.end();
}

int Dialog::PacketLength() const
{
	std::size_t size = this->pages.size() * 3;
	size += this->links.size() * 5;

	UTIL_CFOREACH(this->pages, page)
	{
		size += page.length();
	}

	UTIL_CFOREACH(this->links, link)
	{
		size += link.second.length();
	}

	return size;
}

void Dialog::BuildPacket(PacketBuilder& builder) const
{
	builder.ReserveMore(this->PacketLength());

	UTIL_CFOREACH(this->pages, page)
	{
		builder.AddShort(DIALOG_TEXT);
		builder.AddBreakString(page);
	}

	UTIL_CFOREACH(this->links, link)
	{
		builder.AddShort(DIALOG_LINK);
		builder.AddShort(link.first);
		builder.AddBreakString(link.second);
	}
}

Dialog::~Dialog()
{

}
